/*
  libwftk - Worldforge Toolkit - a widget library
  Copyright (C) 2002 Malcolm Walker <malcolm@worldforge.org>
  Based on code copyright  (C) 1999-2002  Karsten Laux

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA  02111-1307, SA.
*/

#include "questionbox.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "multilineedit.h"
#include "pushbutton.h"
#include "box.h"
#include "filler.h"

#include <sigc++/object_slot.h>
#include <sigc++/bind.h>

void wftk::QuestionBox::packBox(const std::string& question)
{
  getResourceBackground("dialogbox");
  getResourceBackground("questionbox");

  Box *vbox = new Box(Box::TOP_TO_BOTTOM);
  pack(vbox);

  vbox->packBack(new MultiLineEdit(question));

  vbox->packBack(new Filler(Filler::VERTICAL, Font::textFont().metrics().height / 64));

  Box *hbox = new Box(Box::LEFT_TO_RIGHT);
  vbox->packBack(hbox);

  hbox->packBack(new Filler(Filler::HORIZONTAL));

  PushButton* yes = new PushButton("Yes"); 
  hbox->packBack(yes);

  hbox->packBack(new Filler(Filler::HORIZONTAL));

  PushButton* no = new PushButton("No");
  hbox->packBack(no);

  hbox->packBack(new Filler(Filler::HORIZONTAL));

  yes->clicked.connect(SigC::bind(SigC::slot(*this, &QuestionBox::click), true));
  no->clicked.connect(SigC::bind(SigC::slot(*this, &QuestionBox::click), false));
}

void wftk::QuestionBox::click(bool val)
{
  result_ = val;
  result(val);
  close();
}
